<?php

namespace EventPortal\Http\Controllers\Admin\Event;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Event\Actions\DeleteEvent;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\Render;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Domain\Util\Statistics\Facades\Statistics;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Inertia\Response;

class EventController extends Controller
{
    public function __construct()
    {
        $this->middleware('check-permission:event.show')->only('show');
        $this->middleware('check-permission:event.store')->only(['create', 'store']);
        $this->middleware('check-permission:event.update')->only(['edit', 'update']);
        $this->middleware('check-permission:event.destroy')->only('delete');
    }

    public function index(): Response
    {
        $table = Table::make('event.index')
            ->query(Event::query()->isNotArchived()->forAdminInTeam())
            ->request(request())
            ->get();

        $quickButton = Button::make('Create event')
            ->method(InertiaMethod::VISIT())
            ->icon('plus')
            ->route('admin.event.create');

        return Render::component('Event/Index')
            ->key('event.index')
            ->withTable($table)
            ->withQuickButton($quickButton, has_permission('event.store'))
            ->get();
    }

    public function create(): Response
    {
        $form = Form::make('event.create')->model(new Event)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add('Create an event')
            ->get();

        return Render::component('Event/Create')
            ->key('event.create')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->get();
    }

    public function store(Request $request): RedirectResponse
    {
        $payload = Form::make('event.create')
            ->model(new Event)
            ->validate($request->all())
            ->save($request->all());

        return redirect()->route('admin.event.show', [$payload->getModel()]);
    }

    public function edit(Event $event): Response
    {
        $form = Form::make('event.edit')->model($event)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event->id))
            ->add('Edit event')
            ->get();

        return Render::component('Event/Edit')
            ->key('event.edit')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($event->name)
            ->get();
    }

    public function update(Request $request, Event $event): RedirectResponse
    {
        Form::make('event.edit')
            ->model($event)
            ->validate($request->all())
            ->save($request->all());

        return redirect()->route('admin.event.index');
    }

    public function show(Event $event): Response
    {
        $form = Form::make('event.show')->model($event)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name)
            ->get();

        $stats = Statistics::make('events.show')
            ->model($event)
            ->render();

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model($event)
            ->render();

        return Render::component('Event/Show')
            ->key('event.show')
            ->withData($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($event->name)
            ->withStatistics($stats)
            ->withSideBar($sidebar)
            ->get();
    }

    public function delete(Event $event, DeleteEvent $deleteEvent): RedirectResponse
    {
        $deleteEvent->execute($event);

        return redirect()->route('admin.event.index');
    }
}
