<?php

namespace EventPortal\Providers;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Email\Models\Email;
use EventPortal\Domain\Email\Observers\EmailObserver;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Exceptions\Handler;
use EventPortal\Http\Middleware\CheckPermission;
use EventPortal\Http\Middleware\CurrentTeam;
use Illuminate\Contracts\Debug\ExceptionHandler;
use Illuminate\Routing\Router;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\ServiceProvider;
use Inertia\Inertia;

class AppServiceProvider extends ServiceProvider
{
    /** @var string */
    protected $root = __DIR__ . '/../..';

    public function boot(Router $router): void
    {
        $this->app->booted(function (): void {
            $this->loadRoutesFrom("{$this->root}/routes/routes.php");
        });

        $this->loadViewsFrom("{$this->root}/resources/views", 'EventPortal');

        $this->loadMigrationsFrom("{$this->root}/database/migrations");

        $this->loadFactoriesFrom("{$this->root}/database/factories");

        $this->app->bind(ExceptionHandler::class, Handler::class);

        $this->publishes([
            "{$this->root}/tables/delegate.php" => '/tables/delegates.php',
            "{$this->root}/tables/event.php" => '/tables/event.php',
            "{$this->root}/tables/event-delegate.php" => '/tables/event-delegate.php',
        ], 'event-portal-tables');

        $this->publishes([
            "{$this->root}/forms/delegate.php" => '/forms/delegates.php',
            "{$this->root}/forms/event.php" => '/forms/event.php',
            "{$this->root}/forms/event-delegate.php" => '/forms/event-delegate.php',
        ], 'event-portal-forms');

        $this->publishes([
            "{$this->root}/sidebars/event.php" => '/sidebars/event.php',
        ], 'event-portal-sidebars');

        $this->publishes([
            "{$this->root}/statistics/events.php" => '/statistics/events.php',
        ], 'event-portal-statistics');

        $this->publishes([
            "{$this->root}/config/permission.php" => config_path('/permission.php'),
            "{$this->root}/config/state-machine.php" => config_path('/state-machine.php'),
            "{$this->root}/config/audit.php" => config_path('/audit.php'),
            "{$this->root}/config/page-headers.php" => config_path('/page-headers.php'),
            "{$this->root}/config/emails.php" => config_path('/emails.php'),
            "{$this->root}/config/roles.php" => config_path('/roles.php'),
            "{$this->root}/config/permissions.php" => config_path('/permissions.php'),
        ], 'event-portal-config');

        $this->app->bind(ExceptionHandler::class, Handler::class);

        $router->aliasMiddleware('current-team', CurrentTeam::class);
        $router->aliasMiddleware('check-permission', CheckPermission::class);

        Inertia::setRootView('EventPortal::app');

        $this->setSharedData();

        Route::model('event', Event::class);
        Route::model('delegate', Delegate::class);
        Route::model('eventDelegate', EventDelegate::class);
        Route::model('admin', Admin::class);

        Email::observe(EmailObserver::class);
    }

    private function setSharedData(): void
    {
        Inertia::share([
            'navigation' => function () {
                if ($this->viewingAdmin()) {
                    return Sidebar::make('primary')->user(Auth::user())->render()['children'];
                }

                return null;
            },
            'errors' => function () {
                return session('errors') ? session('errors')->getBag('default')->getMessages() : (object) [];
            },
        ]);
    }

    private function viewingAdmin(): bool
    {
        return Auth::check() && Auth::user()->isAdmin() && request()->is(['admin', 'admin/*']);
    }
}
